import dataclasses
import os
from typing import Iterable, Mapping, Optional, Tuple, Union


class Nav:
    """An object representing MkDocs navigation, consisting of files under nested sequences of
    titles, which are treated like paths.
    """

    def __init__(self):
        self._data = {}

    def __setitem__(self, keys: Union[str, Tuple[str, ...]], value: str):
        """Add a link to a file (*value*) into the nav, under the sequence of titles (*keys*).

        For example, writing `nav["Foo", "Bar"] = "foo/bar.md"` would mean creating a nav:
        `{"Foo": {"Bar": "foo/bar.md"}}`.

        Then, writing `nav["Foo", "Another"] = "test.md"` would merge with the existing sections
        where possible:
        `{"Foo": {"Bar": "foo/bar.md", "Another": "test.md"}}`.

        *keys* here can be any non-empty sequence of strings, it's just that Python implicitly
        creates a tuple from the comma-separated items in those square brackets.
        """
        if isinstance(keys, str):
            keys = (keys,)
        cur = self._data
        if not keys:
            raise ValueError(f"The navigation path must not be empty (got {keys!r})")
        for key in keys:
            if not isinstance(key, str):
                raise TypeError(
                    f"The navigation path must consist of strings, but got a {type(key)}"
                )
            if not key:
                raise ValueError(f"The navigation name parts must not be empty (got {keys!r})")
            cur = cur.setdefault(key, {})
        cur[None] = os.fspath(value)

    @dataclasses.dataclass
    class Item:
        level: int
        title: str
        filename: Optional[str]

    def items(self) -> Iterable[Item]:
        return self._items(self._data, 0)

    @classmethod
    def _items(cls, data: Mapping, level: int) -> Iterable[Item]:
        for key, value in data.items():
            if key is not None:
                yield cls.Item(level=level, title=key, filename=value.get(None))
                yield from cls._items(value, level + 1)

    _markdown_escape_chars = tuple("!#()*+-[\\]_`{}")

    def build_literate_nav(self, indentation: Union[int, str] = "") -> Iterable[str]:
        if isinstance(indentation, int):
            indentation = " " * indentation
        for item in self.items():
            line = item.title
            if line.startswith(self._markdown_escape_chars):
                line = "\\" + line
            if item.filename is not None:
                line = f"[{line}]({item.filename})"
            yield indentation + "    " * item.level + "* " + line + "\n"



import logging
import pathlib
import runpy
import tempfile
import urllib.parse

import mkdocs.utils
from mkdocs.config import Config, config_options
from mkdocs.plugins import BasePlugin
from mkdocs.structure.files import Files
from mkdocs.structure.pages import Page

try:
    from mkdocs.exceptions import PluginError
except ImportError:
    PluginError = SystemExit

from . import editor

log = logging.getLogger(f"mkdocs.plugins.{__name__}")
log.addFilter(mkdocs.utils.warning_filter)


class GenFilesPlugin(BasePlugin):
    config_scheme = (("scripts", config_options.Type(list)),)

    def on_files(self, files: Files, config: Config) -> Files:
        self._dir = tempfile.TemporaryDirectory(prefix="mkdocs_gen_files_")

        with editor.FilesEditor(files, config, self._dir.name) as ed:
            for file_name in self.config["scripts"]:
                try:
                    runpy.run_path(file_name)
                except SystemExit as e:
                    if e.code:
                        raise PluginError(f"Script {file_name!r} caused {e!r}")

        self._edit_paths = dict(ed.edit_paths)
        return ed.files

    def on_page_content(self, html, page: Page, config: Config, files: Files):
        repo_url = config.get("repo_url", None)
        edit_uri = config.get("edit_uri", None)

        src_path = pathlib.Path(page.file.src_path)
        if src_path in self._edit_paths:
            path = self._edit_paths.pop(src_path)
            if repo_url and edit_uri:
                # Ensure urljoin behavior is correct
                if not edit_uri.startswith(("?", "#")) and not repo_url.endswith("/"):
                    repo_url += "/"

                page.edit_url = path and urllib.parse.urljoin(
                    urllib.parse.urljoin(repo_url, edit_uri), path
                )

        return html

    def on_post_build(self, config: Config):
        self._dir.cleanup()

        unused_edit_paths = {k: str(v) for k, v in self._edit_paths.items() if v}
        if unused_edit_paths:
            msg = "mkdocs_gen_files: These set_edit_path invocations went unused (the files don't exist): %r"
            log.warning(msg, unused_edit_paths)



"""A magical module that forwards everything to [mkdocs_gen_files.editor.FilesEditor.current][].

Just `import mkdocs_gen_files`.

Then `mkdocs_gen_files.open()` is the same as `mkdocs_gen_files.editor.FilesEditor.current.open()`.
"""

import logging

import mkdocs.utils

from .editor import FilesEditor
from .nav import Nav  # noqa

log = logging.getLogger(f"mkdocs.plugins.{__name__}")
log.addFilter(mkdocs.utils.warning_filter)


def __getattr__(name: str):
    return getattr(FilesEditor.current(), name)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mkdocs_gen_files']

package_data = \
{'': ['*']}

install_requires = \
['mkdocs>=1.0.3,<2.0.0']

entry_points = \
{'mkdocs.plugins': ['gen-files = mkdocs_gen_files.plugin:GenFilesPlugin']}

setup_kwargs = {
    'name': 'mkdocs-gen-files',
    'version': '0.3.5',
    'description': 'MkDocs plugin to programmatically generate documentation pages during the build',
    'long_description': '# mkdocs-gen-files\n\n**[Plugin][] for [MkDocs][] to programmatically generate documentation pages during the build**\n\n[![PyPI](https://img.shields.io/pypi/v/mkdocs-gen-files)](https://pypi.org/project/mkdocs-gen-files/)\n[![GitHub](https://img.shields.io/github/license/oprypin/mkdocs-gen-files)](https://github.com/oprypin/mkdocs-gen-files/blob/master/LICENSE.md)\n[![GitHub Workflow Status](https://img.shields.io/github/workflow/status/oprypin/mkdocs-gen-files/CI)](https://github.com/oprypin/mkdocs-gen-files/actions?query=event%3Apush+branch%3Amaster)\n\n```shell\npip install mkdocs-gen-files\n```\n\n**Continue to the [documentation site][].**\n\n[mkdocs]: https://www.mkdocs.org/\n[plugin]: https://www.mkdocs.org/user-guide/plugins/\n[documentation site]: https://oprypin.github.io/mkdocs-gen-files\n\n## Usage\n\nActivate the plugin in **mkdocs.yml** (`scripts` is a required list of Python scripts to execute):\n\n```yaml\nplugins:\n  - search\n  - gen-files:\n      scripts:\n        - gen_pages.py  # or any other name or path\n```\n\nThen create such a script **gen_pages.py** (this is relative to the root, *not* to the *docs* directory).\n\n```python\nimport mkdocs_gen_files\n\nwith mkdocs_gen_files.open("foo.md", "w") as f:\n    print("Hello, world!", file=f)\n```\n\nThis added a programmatically generated page to our site. That is, the document doesn\'t actually appear in our source files, it only *virtually* becomes part of the site to be built by MkDocs.\n\n**Continue to the [documentation site][].**\n',
    'author': 'Oleh Prypin',
    'author_email': 'oleh@pryp.in',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/oprypin/mkdocs-gen-files',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
